
// Copyright (c) 2007 Ein O'Callaghan
//
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt

#include "stdAfx.hpp"
#include "GhostTracker.hpp"

unsigned WMU_LOG_UINT = ::RegisterWindowMessage(WMU_LOG_UINT_GUID);

unsigned WMU_GHOST_START = NULL;
unsigned WMU_GHOST_STOP = NULL;
unsigned WMU_GHOST_FADE_IN = NULL;
unsigned WMU_GHOST_FADE_OUT = NULL;
unsigned WMU_GHOST_HOOK = NULL;

namespace ghost_tracker 
{

#pragma data_seg(".THESHAREDBITS")

static HWND controler_hwnd = NULL;
static HMODULE h_dll = NULL;

static HHOOK mouse_hook = NULL;
static HWINEVENTHOOK event_hook = NULL;

static HHOOK ghost_hook = NULL;
static DWORD ghost_thread_id = 0;
static DWORD ghost_process_id = 0;

static HWND ghost_traywnd_hwnd = NULL;
static HWND ghost_start_hwnd = NULL;
static HWND ghost_hwnd = NULL;

static bool installed_ = false;

namespace settings
{

double fade_in_alpha = 1;
double fade_out_alpha = 0;
double fade_alpha = fade_in_alpha;

unsigned steps = 20;
unsigned fade_in_time = 70;
unsigned fade_out_time = 300;

}

#pragma data_seg()
#pragma comment(linker, "/section:.THESHAREDBITS,rws")

static HMODULE h_module = NULL;
static UINT_PTR mouse_timer = 0;

static unsigned should_fade_in = 0;

static std::set<HWND> hwnd_set;

enum { FADING_OUT = 1, FADING_IN };
enum { EVENT_FADE = 0x1, MOUSE_FADE = 0x2 };

} // namespace ghost_tracker

void CALLBACK HandleWinEvent(HWINEVENTHOOK hook, DWORD event, HWND hwnd, 
                             LONG idObject, LONG idChild, 
                             DWORD dwEventThread, DWORD dwmsEventTime)
{
	if (::IsZoomed(hwnd) && !::GetParent(hwnd))
	{
		std::pair<std::set<HWND>::iterator, bool> result = ghost_tracker::hwnd_set.insert(hwnd);

		if (result.second && ghost_tracker::hwnd_set.size() == 1) 
			::PostMessage(ghost_tracker::ghost_hwnd, WMU_GHOST_FADE_IN, 141, ghost_tracker::EVENT_FADE);
	}

	for(std::set<HWND>::iterator i=ghost_tracker::hwnd_set.begin(); i!=ghost_tracker::hwnd_set.end(); /**/)
	{
		if (!::IsZoomed(*i))
		{
			i = ghost_tracker::hwnd_set.erase(i);
			if (ghost_tracker::hwnd_set.empty())
				::PostMessage(ghost_tracker::ghost_hwnd, WMU_GHOST_FADE_OUT, 141, ghost_tracker::EVENT_FADE);
		} 
		else 
		{
			++i;
		}
	}
}

VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
{
	POINT point;
	GetCursorPos(&point);

	if (GetWindowThreadProcessId(WindowFromPoint(point), NULL) != ghost_tracker::ghost_thread_id)
	{
		ghost_tracker::mouse_timer = NULL;
		KillTimer(0, idEvent);
		::PostMessage(ghost_tracker::ghost_hwnd, WMU_GHOST_FADE_OUT, 141, ghost_tracker::MOUSE_FADE);
	}
}

LRESULT CALLBACK MouseProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if (nCode < 0) 
		return CallNextHookEx(ghost_tracker::mouse_hook, nCode, wParam, lParam);  

	if (!ghost_tracker::mouse_timer)
	{
		ghost_tracker::mouse_timer = SetTimer(NULL, 0, 100, &TimerProc);
		::PostMessage(ghost_tracker::ghost_hwnd, WMU_GHOST_FADE_IN, 141, ghost_tracker::MOUSE_FADE);
	}

	return CallNextHookEx(ghost_tracker::mouse_hook, nCode, wParam, lParam);
}

void SetDesiredAlpha(BYTE desired_alpha)
{
	if (desired_alpha > 0)
	{	
		SetLayeredWindowAttributes(ghost_tracker::ghost_traywnd_hwnd, NULL, 
			desired_alpha ,LWA_ALPHA);

		if (ghost_tracker::ghost_start_hwnd)
		{	
			::ShowWindow(ghost_tracker::ghost_start_hwnd, SW_SHOWNOACTIVATE);

			BLENDFUNCTION bf = {AC_SRC_OVER, NULL, desired_alpha, AC_SRC_ALPHA};			
			UpdateLayeredWindow(ghost_tracker::ghost_start_hwnd, NULL, NULL, NULL, NULL, NULL, 0, &bf, ULW_ALPHA);
		}
	}
	else
	{
		SetLayeredWindowAttributes(ghost_tracker::ghost_traywnd_hwnd, NULL, 1 ,LWA_ALPHA);

		if (ghost_tracker::ghost_start_hwnd)
			::ShowWindow(ghost_tracker::ghost_start_hwnd, SW_HIDE);
	}
}

LRESULT CALLBACK WindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    if (message == WM_TIMER)
	{
		if (wParam == ghost_tracker::FADING_IN)
		{
			using namespace ghost_tracker::settings;
			double alpha = (fade_in_alpha-fade_out_alpha) / (double)steps;
			fade_alpha += alpha;

			if (fade_alpha >= fade_in_alpha)
			{
				fade_alpha = fade_in_alpha;
				KillTimer(hwnd, ghost_tracker::FADING_IN);
			}

			SetDesiredAlpha((BYTE)(fade_alpha*255));
		}
		else if (wParam == ghost_tracker::FADING_OUT)
		{
			using namespace ghost_tracker::settings;
			double alpha = (fade_in_alpha-fade_out_alpha) / (double)steps;
			fade_alpha -= alpha;

			if (fade_alpha <= fade_out_alpha)
			{
				fade_alpha = fade_out_alpha;
				KillTimer(hwnd, ghost_tracker::FADING_OUT);
			}

			SetDesiredAlpha((BYTE)(fade_alpha*255));
		}
	}
	else if (message == WM_CREATE)
	{
		LONG style = GetWindowLong(ghost_tracker::ghost_traywnd_hwnd, GWL_EXSTYLE) | WS_EX_LAYERED;
		SetWindowLong(ghost_tracker::ghost_traywnd_hwnd, GWL_EXSTYLE, style);

		SetLayeredWindowAttributes(ghost_tracker::ghost_traywnd_hwnd, NULL, 
			(BYTE)(ghost_tracker::settings::fade_alpha*255), LWA_ALPHA);
		
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1101);
    }
	else if (message == WM_DESTROY)
	{
		SetLayeredWindowAttributes(ghost_tracker::ghost_traywnd_hwnd, NULL, 
			(BYTE)(255), LWA_ALPHA);

		LONG style = GetWindowLong(ghost_tracker::ghost_traywnd_hwnd, GWL_EXSTYLE) & ~WS_EX_LAYERED;
		SetWindowLong(ghost_tracker::ghost_traywnd_hwnd, GWL_EXSTYLE, style);

		KillTimer(hwnd, ghost_tracker::FADING_IN);		
		KillTimer(hwnd, ghost_tracker::FADING_OUT);
		
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1102);
    }
	else if (message == WMU_GHOST_START)
	{
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1110);

		if (!ghost_tracker::mouse_hook)
			ghost_tracker::mouse_hook = 
				SetWindowsHookEx(WH_MOUSE, MouseProc, NULL, ghost_tracker::ghost_thread_id);

		if (!ghost_tracker::event_hook)
			ghost_tracker::event_hook = ::SetWinEventHook(EVENT_SYSTEM_MINIMIZESTART, EVENT_OBJECT_LOCATIONCHANGE,
				NULL, &HandleWinEvent, 0, 0, WINEVENT_OUTOFCONTEXT);
	
		using namespace ghost_tracker::settings;
		SetTimer(hwnd, ghost_tracker::FADING_OUT, fade_out_time/steps, NULL);
    }
	else if (message == WMU_GHOST_STOP)
	{
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1111);

		if (ghost_tracker::mouse_hook)
		{
			UnhookWindowsHookEx(ghost_tracker::mouse_hook);
			ghost_tracker::mouse_hook= NULL;
		}

		if (ghost_tracker::event_hook)
		{
			UnhookWinEvent(ghost_tracker::event_hook);
			ghost_tracker::event_hook= NULL;
		}
		
		SetTimer(hwnd, ghost_tracker::FADING_IN, 10, NULL);
    }
	else if (message == WMU_GHOST_FADE_IN)
	{
		ghost_tracker::should_fade_in |= lParam;
		
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, ghost_tracker::should_fade_in);
		
		if (ghost_tracker::should_fade_in)
		{
			KillTimer(hwnd, ghost_tracker::FADING_OUT);

			using namespace ghost_tracker::settings;
			SetTimer(hwnd, ghost_tracker::FADING_IN, fade_in_time/steps, NULL);
		}
    }
	else if (message == WMU_GHOST_FADE_OUT)
	{
		ghost_tracker::should_fade_in &= ~lParam;

		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, ghost_tracker::should_fade_in);

		if (!ghost_tracker::should_fade_in)
		{
			KillTimer(hwnd, ghost_tracker::FADING_IN);

			using namespace ghost_tracker::settings;
			SetTimer(hwnd, ghost_tracker::FADING_OUT, fade_out_time/steps, NULL);
		}
    }
	else
	{
        return DefWindowProc(hwnd, message, wParam, lParam);
	}
    
    return 0;
}

BOOL APIENTRY DllMain(HMODULE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
	ghost_tracker::h_module = hModule;	

	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		::DisableThreadLibraryCalls((HINSTANCE) hModule);

		if (WMU_GHOST_HOOK == NULL)
			WMU_GHOST_HOOK = ::RegisterWindowMessage(L"WMU_GHOST_HOOK_{6F8B1B93-7096-4a24-8AE5-D1F1DF5E4AC7}" );	

		if (WMU_GHOST_START == NULL)
			WMU_GHOST_START = ::RegisterWindowMessage(L"WMU_GHOST_START_{6F8B1B93-7096-4a24-8AE5-D1F1DF5E4AC7}");
		if (WMU_GHOST_STOP == NULL)
			WMU_GHOST_STOP = ::RegisterWindowMessage(L"WMU_GHOST_STOP_{6F8B1B93-7096-4a24-8AE5-D1F1DF5E4AC7}");
		if (WMU_GHOST_FADE_IN == NULL)
			WMU_GHOST_FADE_IN = ::RegisterWindowMessage(L"WMU_GHOST_FADE_IN_{6F8B1B93-7096-4a24-8AE5-D1F1DF5E4AC7}");
		if (WMU_GHOST_FADE_OUT == NULL)
			WMU_GHOST_FADE_OUT = ::RegisterWindowMessage(L"WMU_GHOST_FADE_OUT_{6F8B1B93-7096-4a24-8AE5-D1F1DF5E4AC7}");

		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1001);
		break;
	case DLL_THREAD_ATTACH:
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1002);
		break;
	case DLL_THREAD_DETACH:
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1003);
		break;
	case DLL_PROCESS_DETACH:
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1004);
		break;
	}
    return TRUE;
}

LRESULT CALLBACK CallWndProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if (nCode < 0) 
		return CallNextHookEx(ghost_tracker::ghost_hook, nCode, wParam, lParam);

	CWPSTRUCT* pCW = (CWPSTRUCT*)lParam;

	if ((pCW->message == WMU_GHOST_HOOK) && pCW->lParam) 
	{
		if (!::UnhookWindowsHookEx(ghost_tracker::ghost_hook))
		{
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 2004);
			goto hook_callback_end;
		}

		if (ghost_tracker::installed_)
		{
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 2005);
			goto hook_callback_end;
		}

		wchar_t lib_name[MAX_PATH]; 
		::GetModuleFileName(ghost_tracker::h_module, lib_name, MAX_PATH);
		ghost_tracker::h_dll = ::LoadLibrary(lib_name);

		if (!ghost_tracker::h_dll)
		{
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 2003);
			goto hook_callback_end;
		}

		WNDCLASSEX wincl;

		wincl.hInstance = ghost_tracker::h_module;
		wincl.lpszClassName = L"ghostClass";
		wincl.lpfnWndProc = WindowProc;
		wincl.style = NULL;
		wincl.cbSize = sizeof(WNDCLASSEX);
		wincl.hIcon = NULL;
		wincl.hIconSm = NULL;
		wincl.hCursor = NULL;
		wincl.lpszMenuName = NULL;
		wincl.cbClsExtra = 0;
		wincl.cbWndExtra = 0;
		wincl.hbrBackground = NULL;

		if (!RegisterClassEx(&wincl))
		{
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 2001);
		}

		if (!ghost_tracker::ghost_hwnd)
		{
			ghost_tracker::ghost_hwnd = CreateWindowEx (
				   0,
				   L"ghostClass",
				   NULL,
				   0, 0, 0, 0, 0,
				   HWND_MESSAGE,
				   NULL,
				   ghost_tracker::h_module,
				   NULL
				   );
		}

		if (!ghost_tracker::ghost_hwnd)
		{
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 2002);
			goto hook_callback_end;
		}

		ghost_tracker::installed_ = true;

		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1300);	
	}
	else if (pCW->message == WMU_GHOST_HOOK) 
	{
		if (!::UnhookWindowsHookEx(ghost_tracker::ghost_hook))
		{
			ghost_tracker::ghost_hook = NULL;
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 2007);
		}

		if (!ghost_tracker::installed_)
		{
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 2008);
			goto hook_callback_end;
		}

		if (::DestroyWindow(ghost_tracker::ghost_hwnd))
		{
			ghost_tracker::ghost_hwnd = NULL;
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1402);
		}

		if (::FreeLibrary(ghost_tracker::h_dll))
		{
			ghost_tracker::h_dll = NULL;
			::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1403);
		}

		ghost_tracker::installed_ = false;
	}

hook_callback_end:
	return CallNextHookEx(ghost_tracker::ghost_hook, nCode, wParam, lParam);
}

namespace ghost_tracker
{

WNDTRACKER_API void set_controler_hwnd(HWND controler) 
{ 
	controler_hwnd = controler;
	::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 1000);
}

WNDTRACKER_API HHOOK install_ghost()
{
	ghost_traywnd_hwnd = FindWindowEx(NULL, NULL, L"Shell_TrayWnd", L"");
	ghost_start_hwnd = FindWindowEx(NULL, NULL, L"Button", L"Start");
	ghost_thread_id = GetWindowThreadProcessId(ghost_traywnd_hwnd, &ghost_process_id);

	ghost_hook = SetWindowsHookEx(WH_CALLWNDPROC, &CallWndProc, h_module, ghost_thread_id);
	
	if (ghost_hook)
		::SendMessage(ghost_traywnd_hwnd, WMU_GHOST_HOOK, 0, true);
	else
		::PostMessage(ghost_tracker::controler_hwnd, WMU_LOG_UINT, 141, 2006);

	return ghost_hook;
}

WNDTRACKER_API void uninstall_ghost()
{
	if (ghost_traywnd_hwnd)
	{
		::PostMessage(ghost_hwnd, WMU_GHOST_STOP, 0, 0);

		ghost_hook = SetWindowsHookEx(WH_CALLWNDPROC, &CallWndProc, h_module, ghost_thread_id);
		::SendMessage(ghost_traywnd_hwnd, WMU_GHOST_HOOK, 0, false);
	}
}

WNDTRACKER_API HWND get_GHOST_hwnd()
{	
	return ghost_hwnd;
}

WNDTRACKER_API bool ghost_installed()
{
	return installed_;
}

WNDTRACKER_API bool start_ghost()
{	
	::PostMessage(ghost_hwnd, WMU_GHOST_START, 0, 0);

	return true;
}

WNDTRACKER_API bool stop_ghost()
{	
	::PostMessage(ghost_hwnd, WMU_GHOST_STOP, 0, 0);	
	::Sleep(ghost_tracker::settings::fade_in_time*2);

	return true;
}

} // namespace ghost_tracker
