
// Copyright (c) 2007 Ein O'Callaghan
//
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt

#include "stdAfx.hpp"
#include "PropertyDlg.hpp"
#include "../GhostTracker/GhostTracker.hpp"

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	po::options_description desc("To call the program with parameters create a shortcut file and in the properties add the desired parameters to the end of the \"Target:\" setting. \n\n The available parameters are");
	desc.add_options()
		("no-gui", "Display no GUI")
		("start", "Start the fader")
		("stop", "Stop the fader\n")
		("install", "Install the dll")
		("uninstall", "Uninstall the dll")
		("enable", "Enable tracking events")
		("disable", "Disable tracking events\n")
		("active-alpha", po::value<double>(&ghost_tracker::settings::fade_in_alpha)->default_value(1), "Active taskbar alpha (0-1)")
		("inactive-alpha", po::value<double>(&ghost_tracker::settings::fade_out_alpha)->default_value(0), "Inactive alpha (0-1)")
		("fade-in-time", po::value<unsigned>(&ghost_tracker::settings::fade_in_time)->default_value(80), "Time to fade in (ms)")
		("fade-out-time", po::value<unsigned>(&ghost_tracker::settings::fade_out_time)->default_value(300), "Time to fade out (ms)")
		("steps", po::value<unsigned>(&ghost_tracker::settings::steps)->default_value(20), "Higher = Smoother\n")
		("debug", "Display the Debug tab");

	try
	{

	po::variables_map vm;
	std::vector<wstring> args = po::split_winmain(GetCommandLineW());
	po::store(po::wcommand_line_parser(args).options(desc).run(), vm);
	po::notify(vm);  
		
	if (vm.count("install")) 
	{
		ghost_tracker::install_ghost();
	} 
			
	if (vm.count("start")) 
	{
		ghost_tracker::install_ghost();
		ghost_tracker::start_ghost();
	} 

	if (vm.count("enable")) 
	{
		ghost_tracker::start_ghost();
	} 

	if (vm.count("disable")) 
	{
		ghost_tracker::stop_ghost();
	} 

	if (vm.count("stop")) 
	{
		ghost_tracker::stop_ghost();
		ghost_tracker::uninstall_ghost();
	} 

	if (vm.count("uninstall")) 
	{
		ghost_tracker::uninstall_ghost();
	} 

	if (vm.count("no-gui") == 0) 
	{	
		std::stringstream description;
		description << desc;

		string desc_str = description.str();
		boost::replace_all(desc_str, "\n", "\r\n");

		bool debug = vm.count("debug") != 0;
		PropertyDlg pDlg(L"Ghostbar", aux::from_utf8(desc_str), debug);
		pDlg.DoModal();
    }

	}
	catch (const std::exception& e)
	{
		std::stringstream description;
		description << "Caught exception : " << e.what() << "\n\n";
		description << desc;

		string desc_str = description.str();
		boost::replace_all(desc_str, "\n", "\r\n");

		MessageBox(0, (aux::from_utf8(desc_str)).c_str(), L"Some Sort of Problem", 0);
	}
}
