
// Copyright (c) 2007 Ein O'Callaghan
//
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt

#include "stdAfx.hpp"
#include "PropertyDlg.hpp"

#include "../ghostTracker/ghostTracker.hpp"

static std::set<HWND> hwnd_set;

unsigned WMU_LOG_UINT = ::RegisterWindowMessage(WMU_LOG_UINT_GUID);

PropertyDlg::PropertyDlg(LPCTSTR title, wstring description, bool debug, UINT uStartPage, HWND hWndParent) :
    CPropertySheet(title, uStartPage, hWndParent),
	propSettings(description),
	m_bCentered(false)
{
	AddPage(propSettings);
	if (debug) AddPage(propOptions);
}

PropertyDlg::~PropertyDlg(void)
{}

BOOL PropertiesOptions::OnInitDialog(HWND hwndFocus, LPARAM lParam)
{
	log_list_.Attach(GetDlgItem(IDC_LOG));

	ghost_tracker::set_controler_hwnd(*this);

	if (ghost_tracker::ghost_installed())
		PostMessage(WMU_LOG_UINT, 314, 1001);
	else
		PostMessage(WMU_LOG_UINT, 314, 1002);

    return DoDataExchange(false);
}

void PropertiesOptions::OnClose()
{}

void PropertiesOptions::OnInstall(UINT, int, HWND)
{
	ghost_tracker::install_ghost();
}

void PropertiesOptions::OnUninstall(UINT, int, HWND)
{
	ghost_tracker::uninstall_ghost();
}

void PropertiesOptions::OnEnable(UINT, int, HWND)
{
	ghost_tracker::start_ghost();
}

void PropertiesOptions::OnDisable(UINT, int, HWND)
{
	ghost_tracker::stop_ghost();
}

int PropertiesOptions::OnApply()
{
	return DoDataExchange(true);
}

LRESULT PropertiesOptions::OnLogUint(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{	
	int index = log_list_.AddString(
		(wformat(L"Uint: %1%, %2%") % (unsigned)wParam % (unsigned)lParam).str().c_str());

	return 0;
}

BOOL PropertiesSettings::OnInitDialog(HWND hwndFocus, LPARAM lParam)
{
	SetDlgItemText(IDC_DESCRIPTION, description_.c_str());

	if (CFont font = GetFont())
	{
		CLogFont lf;		
		if (font.GetLogFont(&lf))
		{
			lf.lfWidth = 6;
			lf.lfPitchAndFamily = FIXED_PITCH;
			lf.lfFaceName[0] = 0;

			font_fixedw_.CreateFontIndirect(&lf);
		}
	}

	CWindow wndControl = GetDlgItem(IDC_DESCRIPTION);
	wndControl.SetFont(font_fixedw_);

    return DoDataExchange(false);
}

void PropertiesSettings::OnClose()
{}

void PropertiesSettings::OnStart(UINT, int, HWND)
{
	ghost_tracker::install_ghost();
	ghost_tracker::start_ghost();
}

void PropertiesSettings::OnStop(UINT, int, HWND)
{
	ghost_tracker::stop_ghost();
	ghost_tracker::uninstall_ghost();
}

int PropertiesSettings::OnApply()
{
	return DoDataExchange(true);
}