
// Copyright (c) 2007 Ein O'Callaghan
//
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt

#ifndef GLOBAL_STRING_CONV
#define GLOBAL_STRING_CONV

#include <string>
#include <boost/array.hpp>
#include "unicode.hpp"

namespace aux
{

inline std::wstring mbstowcs(const std::string& ustr) 
{
	std::wstring wstr;
	
	unicode::transcode<unicode::utf8, unicode::wchar_encoding>(
	   ustr.begin(),
	   ustr.end(),
	   std::insert_iterator<std::wstring>(wstr, wstr.end())
	);

	return wstr;
}

inline std::string wcstombs(const std::wstring& wstr) 
{	
	std::string ustr;

	unicode::transcode<unicode::wchar_encoding, unicode::utf8>(
	   wstr.begin(),
	   wstr.end(),
	   std::insert_iterator<std::string>(ustr, ustr.end())
	);

	return ustr;
}

inline std::string wstr_to_str(const std::wstring& wstr) 
{ 		
	std::string str;
	str.reserve(wstr.length());
	
	for (std::string::const_iterator i=str.begin(); i!=str.end(); ++i)
	{
		wchar_t wide_char = *i;
		char narrow_char = 0;
		
		narrow_char = *(reinterpret_cast<char*>(&wide_char));
		str.push_back(narrow_char);
	}
	
	return str;
}

inline std::wstring str_to_wstr(const std::string& str) 
{ 		
	std::wstring wstr;
	wstr.reserve(str.length());
	
	for (std::string::const_iterator i=str.begin(); i!=str.end(); ++i)
	{
		char narrow_char = *i;
		wchar_t wide_char = 0;
		
		*(reinterpret_cast<char*>(&wide_char)) = narrow_char;
		wstr.push_back(wide_char);
	}
	
	return wstr;
}

inline std::string to_utf8(const std::wstring& str) 
{ 
	return wcstombs(str); 
}

inline std::wstring from_utf8(const std::string& str) 
{ 
	return mbstowcs(str);  
}

} // namespace aux

#endif // GLOBAL_STRING_CONV